from epiphany.evaluators.open_ai_completion import OpenAICompletion


class IntimateQuestionGenerator:
    """
    This class generates a

    Attributes:
        openAIcompletion (OpenAICompletion): Instance for interactions with OpenAI's API.
    """

    SYSTEM_MESSAGE = """
        You excel at crafting personalized cold emails that assist students in reaching out to potential managers, inquiring further about the use-cases of the company.
    """

    USER_MESSAGE_TEMPLATE = """
        Let's think step by step.
        1. Consider the following:
        name of potential manager: {}
        role: {}.
        potential projects aabout the company: {}.
        2. For research purpose, generate an intimate and brief email 'email' for the person that a student could ask to show interest and intimacy while asking information about their use-cases.
        3. Return a JSON object in the following format: "email": 'email'.
    """

    def __init__(self, model, open_ai_key):
        """
        Initialize the QuestionAnswerer class.
        """
        self.openAIcompletion = OpenAICompletion(model, open_ai_key)

    def generate(self, name: str, role: str, context: str):
        """
        Evaluation for is response faithful to context
        """
        user_message = self.USER_MESSAGE_TEMPLATE.format(name, role, context)
        system_message = self.SYSTEM_MESSAGE
        message = [
            {"role": "system", "content": system_message},
            {"role": "user", "content": user_message},
        ]

        openai_response = self.openAIcompletion.get_completion_from_messages(message)
        openai_response_json = self.openAIcompletion.extract_json_from_response(openai_response)
        question = openai_response_json["email"]
        return question
